#!/usr/bin/env python3

from mrtopo.logger import log
from mrtopo.structures import config, Topology
from mrtopo.mutator import mutate
from mrtopo.translator import c_read, p_read, m_write, desc_write, list_write
from mrtopo.interpreter import interpret
from mrtopo.util.filetype import FileType
from mrtopo.validator.validator import validate
import os

__version__ = "0.1.0"

def main_routine(args):

    log('MrTopo.v.' + __version__ + '>')

    _file, file_type = args

    if file_type == FileType.CONFIG:
        configuration = config.Config()  # initialize default config

        # translate
        if _file:
            configuration = c_read(_file)

        # interpret
        interpret(configuration)

    elif file_type == FileType.PYTHON:

        pyfile = None

        # translate
        if _file:
            pyfile = p_read(_file)

        network = None
        if pyfile:
            # interpret
            network = interpret(pyfile)

        mutant_networks = mutate(network)

        for mutant in mutant_networks:
            m_write(mutant, _file)

        desc_write(mutant_networks)


def validate_routine(args, name=None, long=False):
    log('MrTopo.v.' + __version__ + '-validator>')

    dest, file_type = args

    files = []

    if file_type == FileType.DIRECTORY:
        dir = os.listdir(dest)
        for _file in dir:
            n = len(_file)
            if _file[n-2] + _file[n-1] == "py":
                files.append(f'{dest}/{_file}')
    else:
        files.append(dest)

    descriptor = []

    for _file in files:
        descriptor.append(validate(_file, name, long))

    list_write(descriptor, "validator.txt")



import click
from mrtopo.__main__ import main_routine, validate_routine
from mrtopo.util.filetype import FileType

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group()
def cli():
    '''
    MrTopo is a python application that generates mutant Mininet topology files for network testing purposes.
    To get information on specific subcommands, use `mrtopo [COMMAND] --help`.
    '''
    pass


@click.group()
def mutator():
    '''
    MrTopo mutator
    '''
    pass


@click.command()
@click.option('-f', '--file', type=str, required=True, help="Python file that MrTopo should mutate.")
def python_file(file):
    '''
    Mutate a Mininet python file.
    '''
    if file != "":
        main_routine((file, FileType.PYTHON))


@click.command()
@click.option('-f', '--file', type=str, required=True, help="MrTopo config file.")
def config_file(file):
    '''
    Generate mutations from a configuration file.
    '''
    if file != "":
        click.echo("This option has not been implemented yet!")


@click.group()
def validator():
    pass


@click.command()
@click.option('-f', '--file', type=str, help="Validate a Mininet topology python file.")
@click.option('-t','--topology-name', type=str, default=None,
              help="Name of topology found in python file.\nExample: \'topos = { 'someName': ... } - someName would "
                   "be the topology-name. Only use this option if you know the topology name.")
@click.option('--long/--not-long', default=False,
              help="Long test flag (i.e. pingall)")
def validate_file(file, topology_name, long):
    '''
    Validate a Mininet topology python file.
    '''
    if file != "":
        validate_routine((file, FileType.PYTHON), topology_name, long)


@click.command()
@click.option('-d', '--dir', type=str, help="Validate a directory of Mininet topologies.")
@click.option('-t','--topology-name', type=str, default=None,
              help="Name of topology found in python file.\nExample: \'topos = { 'someName': ... } - someName would "
                   "be the topology-name. Only use this option if you know that all python files in the specified dir"
                   "follow the topology name given.")
@click.option('--long/--not-long', default=False,
              help="Long test flag (i.e. pingall)")
def validate_dir(dir, topology_name, long):
    '''
    Validate a directory of Mininet topologies.
    '''
    if dir != "":
        validate_routine((dir, FileType.DIRECTORY), topology_name, long)


# build mutator
mutator.add_command(python_file)
mutator.add_command(config_file)

# build validator
validator.add_command(validate_file)
validator.add_command(validate_dir)

# build cli
cli = click.CommandCollection(sources=[mutator, validator])

# DEBUG

if __name__ == '__main__':
    cli()



from enum import Enum
from mrtopo.structures.mutantnetwork import MutantNetwork
from mrtopo.structures.network import Network
import random
import copy


class Operations(Enum):
    ADDLINK = 0
    ADDSWITCH = 1
    REMOVELINK = 2
    REMOVESWITCH = 3


def do(operation: Operations, network: Network, id: int):
    if operation == Operations.ADDLINK:
        return add_link(network, id)
    if operation == Operations.ADDSWITCH:
        return add_switch(network, id)
    if operation == Operations.REMOVELINK:
        return remove_link(network, id)
    if operation == Operations.REMOVESWITCH:
        return remove_switch(network, id)


def add_link(network: Network, id):
    description = "Added link"

    found = False

    s1 = None
    s2 = None

    attempts = 0

    not_attempted = copy.deepcopy(list(network.network.keys()))

    while not found:
        if attempts > len(network.switches):
            # ALL NODES LINKED TOGETHER
            # TODO ERROR
            return None

        s1 = random.choice(not_attempted)
        already_linked = network.network[s1]

        if len(already_linked) >= len(network.switches):
            not_attempted.remove(s1)
            continue
        else:
            not_attempted.remove(s1)

            tmp_arr = copy.deepcopy(not_attempted)

            for linked in already_linked:
                if linked in tmp_arr:
                    tmp_arr.remove(linked)

            s2 = random.choice(tmp_arr)

            if not network.link_exists(s1, s2):
                break

        attempts += 1

    network.add_link(s1, s2)

    description += " " + str([s1, s2])

    mn = MutantNetwork(network, id, description, Operations.ADDLINK)
    mn.add_modified_item([s1, s2])
    return mn


def add_switch(network: Network, id):
    description = "Added switch"

    added_switch = "MRTOPO_S"

    description += " " + added_switch

    random_switch = random.choice(list(network.network.keys()))  # choose switch to add link

    network.add_switch(added_switch)

    description += " and added link to " + random_switch

    network.add_link(added_switch, random_switch)  # add link

    mn = MutantNetwork(network, id, description, Operations.ADDSWITCH)
    mn.add_modified_item([added_switch, random_switch])
    return mn


def remove_link(network: Network, id):
    description = "Removed link"

    removed_link = random.choice(network.links)

    description += " " + str(removed_link)

    network.remove_link(removed_link[0], removed_link[1])

    mn = MutantNetwork(network, id, description, Operations.REMOVELINK)
    mn.add_modified_item(removed_link)
    return mn


def remove_switch(network: Network, id):
    description = "Removed switch"

    removed_switch = random.choice(network.switches)

    description += " " + str(removed_switch)

    network.remove_switch(removed_switch)

    mn = MutantNetwork(network, id, description, Operations.REMOVESWITCH)
    mn.add_modified_item(removed_switch)
    return mn



"""
    MrTopo Validator - validates MrTopo mutated topologies
"""

import os
import subprocess

from mrtopo.logger import log


def validate(file, name=None, long=False):
    log(f"Validating: {file}")

    if not name:
        name = extract_name(file)

    log(f"Topology name: {name}")

    build = construct(file, name)
    results = test(file, name, long)

    log(f"Success!")

    descriptor = f"Build Details:\n{build}\n"

    for res in results:
        descriptor += f"{res}\n"

    return bundle(file, descriptor)


def construct(file, name):
    if os.geteuid() != 0:
        log(f"Root access is required to continue the execution of this script.")
    res = call(f'sudo mn --custom {file} --topo {name}')
    call('exit')
    return res


def test(file, name, long):
    benchmark = f"Benchmark:\n{call(f'sudo mn --custom {file} --topo {name} --test none')}"
    if long:
        pingall = f"Pingall:\n{call(f'sudo mn --custom {file} --topo {name} --test pingall')}"
        return [benchmark, pingall]
    else:
        return [benchmark]


def extract_name(file):
    with open(file, 'r') as f:
        _f = f.readlines()

    for line in _f:
        if line.__contains__('topos') and line.__contains__('{'):
            if line.__contains__("\'"):
                return line.split("\'")[1]
            elif line.__contains__('\"'):
                return line.split('\"')


def bundle(file, descriptor):
    return f"Bundle {file}:\n{descriptor}\n"

def call(command):
    try:
        return log(subprocess.check_output(command, shell=True).decode('utf-8'))
    except subprocess.CalledProcessError:
        return log(f'Command: {command} - failed')


import setuptools
import re

with open("README.md", "r") as fh:
    long_description = fh.read()

version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open('mrtopo/__main__.py').read(),
    re.M
    ).group(1)

setuptools.setup(
    name='mrtopo',
    version=version,
    packages=setuptools.find_packages(),
    url='https://github.com/FaizChishtie/mrtopo',
    license='MIT',
    author='faizchishtie',
    author_email='faizchishtie@gmail.com',
    description='Mutate Mininet topology files with MrTopo',
    python_requires='>=3.0',
    entry_points={'console_scripts': ['mrtopo = mrtopo.cli:cli']},
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
            'mininet', 'click'
    ],
    keywords='topology network startup',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Utilities',
        'Typing :: Typed',
    ],
)
